﻿# vercheck.ps1 - Powershell script to Check for Compusource application requirements
# Rev# CC.UT.06.10.00    Last Mod: 09/19/17

<# Instructions:
Run the vercheck.bat from /compu/UTIL
Right click on vercheck.bat and select run as Administrator
Output will be stored in vercheck.log

Script will log the following:
	Installed applications.
	Versions of
		ubl.exe
		usqlsd32.exe
		cfaappserver.exe
		metalcentric.exe
		SQL Server instances
		SQL Report Server Services
	Free disk space
#>
$computername = $env:computername

$sqlarray = @()
$ublarray = @()
$rsarray = @()

Write-Output "Checkvers Starting: $(Get-Date)"
Write-Output "=============================================="
Write-Output "*** Checking for installed Programs ***"

#Define the variable to hold the location of Currently Installed Programs
$UninstallKey="SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall"

#Create an instance of the Registry Object and open the HKLM base key
$reg=[microsoft.win32.registrykey]::OpenRemoteBaseKey('LocalMachine',$computername)

#Drill down into the Uninstall key using the OpenSubKey Method
$regkey=$reg.OpenSubKey($UninstallKey)

#Retrieve an array of string that contain all the subkey names
$subkeys=$regkey.GetSubKeyNames()

#Open each Subkey and use GetValue Method to return the required values for each
foreach($key in $subkeys){

        $thisKey=$UninstallKey+"\\"+$key
        $thisSubKey=$reg.OpenSubKey($thisKey)
        $obj = New-Object PSObject
        $obj | Add-Member -MemberType NoteProperty -Name "ComputerName" -Value $computername
        $obj | Add-Member -MemberType NoteProperty -Name "DisplayName" -Value $($thisSubKey.GetValue("DisplayName"))
        $obj | Add-Member -MemberType NoteProperty -Name "DisplayVersion" -Value $($thisSubKey.GetValue("DisplayVersion"))
        $obj | Add-Member -MemberType NoteProperty -Name "InstallLocation" -Value $($thisSubKey.GetValue("InstallLocation"))
        $obj | Add-Member -MemberType NoteProperty -Name "Publisher" -Value $($thisSubKey.GetValue("Publisher"))
        $sqlarray += $obj
    }


# Display these results:
$sqlarray | Where-Object { $_.DisplayName } | select ComputerName, DisplayName, DisplayVersion, Publisher | ft -AutoSize | Out-String -Width 132


# Check for other versions of requirements:
Write-Output "======================================================"
Write-Output "*** Checking for Transoft and Compusource Programs ***"
# Get location of UBL Executible:
$ublEnvKey="SOFTWARE\\Wow6432Node\\Transoft\\UBL\\2.0\\Environment\\"

#Drill down into the UBL key using the OpenSubKey Method
$ubl=$reg.OpenSubKey($ublEnvKey)
# Get the UBL.EXE location
$ublLoc=$ubl.GetValue("UBLNTEXE")

# Compu should be on the same drive as UBL
$ublDrv=$ublLoc[0]
$compu=$ublDrv+":\Compu"

# Build list of files to check
$userFiles=($ublLoc,"C:\MCFile\MCFILE.exe","C:\Program Files (x86)\Transoft\TCF\Server\cfaappserver.exe","$compu\USQLCS\Bin\usqlsd32.exe","${ublDrv}:\USQLCS\Bin\usqlsd32.exe","C:\Program Files (x86)\Compusource\Metalcentric\Metalcentric.exe")

foreach($userfile in $userFiles)
{

    if (Test-Path $userfile) {
            #(dir $userfile).VersionInfo | fl
        $obj2 = New-Object PSObject
        $obj2 | Add-Member -MemberType NoteProperty -Name "ProductName" -Value (dir $userfile).VersionInfo.ProductName
        $obj2 | Add-Member -MemberType NoteProperty -Name "FileName" -Value (dir $userfile).VersionInfo.FileName
        $obj2 | Add-Member -MemberType NoteProperty -Name "FileVersion" -Value (dir $userfile).VersionInfo.FileVersion
        $obj2 | Add-Member -MemberType NoteProperty -Name "ProductVersion" -Value (dir $userfile).VersionInfo.ProductVersion
        $ublarray += $obj2
    } else {
    Write-Output "==========================="
    Write-Output "*** $userfile not found ***"
    Write-Output "==========================="
    }
}

# Display these results:
$ublarray | select ProductName, FileName, FileVersion, ProductVersion | ft -AutoSize | Out-String -Width 132

# Check SQL Server
Write-Output "========================================================"
Write-Output "Gathering SQL information - Current Server $computername"

if (Test-Connection -ComputerName $computername -Count 1)
	{
		Write-Output "==============================="
        Write-Output "Processing Server $computername"
        Write-Output "==============================="
        $regKey = [Microsoft.Win32.RegistryKey]::OpenRemoteBaseKey([Microsoft.Win32.RegistryHive]::LocalMachine, $computername)
        Write-Output "Checking for 32-bit versions of SQL Server"
		$SqlKey = $regKey.OpenSubKey("SOFTWARE\Wow6432Node\Microsoft\Microsoft SQL Server\Instance Names\SQL")

		if ($SqlKey -ne $null)
		{
			Foreach ($instance in $SqlKey.GetValueNames())
			{

				$SQLServerObject = New-Object PSObject
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name ServerName -Value $computername
				$InstanceName = $SqlKey.GetValue("$instance")
				Write-Output "Parsing SQL-Server Information - Current Instance $instancename"
				$InstanceKey = $regKey.OpenSubkey("SOFTWARE\Wow6432Node\Microsoft\Microsoft SQL Server\$InstanceName\Setup")
				$FileVersion = $($($(dir "\\$computername\$($InstanceKey.GetValue("SQLBinRoot").ToString().Replace(":","$"))\sqlservr.exe").VersionInfo.FileVersion).Split(" ")[0] -replace "(?<!\d)0+(?=\d+)")
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name InstanceName -Value $InstanceName
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name Edition -Value $InstanceKey.GetValue("Edition")
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name Version -Value $InstanceKey.GetValue("Version")
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name FileVersion -Value $FileVersion
				$SQLServerObject

            }
			Write-Output "Parsing 32-bit SQL-Server Information completed"
            Write-Output "========================================================"

		}
		else
		{
			Write-Output "The Server $computername doesn't have 32-bit SQL Server installed"
            Write-Output "================================================================="
		}
        Write-Output "Checking for 64-bit versions of SQL Server"
        Write-Output "=========================================="
		$SqlKey = $regKey.OpenSubKey("SOFTWARE\Microsoft\Microsoft SQL Server\Instance Names\SQL")

		if ($SqlKey -ne $null)
		{
			Foreach ($instance in $SqlKey.GetValueNames())
			{

				$SQLServerObject = New-Object PSObject
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name ServerName -Value $computername
				$InstanceName = $SqlKey.GetValue("$instance")
				Write-Output "Parsing SQL-Server Information - Current Instance $instancename"
				$InstanceKey = $regKey.OpenSubkey("SOFTWARE\Microsoft\Microsoft SQL Server\$InstanceName\Setup")
				$FileVersion = $($($(dir "\\$computername\$($InstanceKey.GetValue("SQLBinRoot").ToString().Replace(":","$"))\sqlservr.exe").VersionInfo.FileVersion).Split(" ")[0] -replace "(?<!\d)0+(?=\d+)")
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name InstanceName -Value $InstanceName
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name Edition -Value $InstanceKey.GetValue("Edition")
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name Version -Value $InstanceKey.GetValue("Version")
				$SQLServerObject | Add-Member -MemberType NoteProperty -Name FileVersion -Value $FileVersion
				$SQLServerObject

            }
			Write-Output "Parsing 64-bit SQL-Server Information completed"
            Write-Output "==============================================="

		}
		else
		{
			Write-Output "The Server $computername doesn't have 64-bit SQL Server installed"
            Write-Output "================================================================="
		}


	}
	else
	{
		Write-Output  "Could not connect to SQL Service on $computername"
        Write-Output "=================================================="
	}


# Display these results:
$SQLServerObject | Where-Object { $_.DisplayName } |select InstanceName, Edition, Version, FileVersion| ft -AutoSize | Out-String -Width 132

Write-Output "========================================================"
Write-Output  "Checking for other SQL Products"
Write-Output "========================================================"
get-wmiobject win32_product |
where {$_.Name -match "SQL" -AND $_.vendor -eq "Microsoft Corporation"} |
select name, version

Write-Output "========================================================"
Write-Output  "Checking for other SQL Services"
Write-Output "========================================================"
Get-Service *SQL*

Write-Output "========================================================"
Write-Output  "Checking for SQL Report Server Services"
Write-Output "========================================================"
$svrlst=Get-Service *Report*

foreach($svr in $svrlst){

        $obj3 = New-Object PSObject
        $obj3 | Add-Member -MemberType NoteProperty -Name "ServiceName" -Value $($svr.Name)
        $obj3 | Add-Member -MemberType NoteProperty -Name "DisplayName" -Value $($svr.DisplayName)
        $obj3 | Add-Member -MemberType NoteProperty -Name "Status" -Value $($svr.Status)
        $rsarray += $obj3
    }


# Display these results:
if ($rsarray -ne $null){
    $rsarray | ft -AutoSize | Out-String -Width 132
} else {
    Write-Output "SQL Reporting Service not installed."
}

Write-Output "========================================================"
Write-Output  "Checking Disk Space"
Write-Output "========================================================"

$disks = Get-WmiObject Win32_LogicalDisk
foreach ($disk in $disks) {
  if ($disk.Size -ne $null) {
  $disk | select DeviceID, @{Name="Disk Size (MB)";Expression={"{0:N0}" -F ($_.Size / 1MB)}}, @{Name="Free Space (MB)";Expression={"{0:N0}" -F ($_.FreeSpace / 1MB)}} | ft -AutoSize | Out-String -Width 132
  }
}

Write-Output "========================================================"
Write-Output "Checkvers Completed: $(Get-Date)"
Write-Output "========================================================"

